#ifndef __SHAPEANALYSIS_CXX__
#define __SHAPEANALYSIS_CXX__

#include "ShapeAnalysis.h"
#include "LArOpenCV/ImageCluster/AlgoClass/PixelChunk.h"
#include "LArOpenCV/ImageCluster/AlgoData/Vertex.h"
#include "Geo2D/Core/Geo2D.h"
#include <cassert>

namespace larocv {
  
  /// Global larocv::ShapeAnalysisFactory to register AlgoFactory
  static ShapeAnalysisFactory __global_ShapeAnalysisFactory__;
  
  void ShapeAnalysis::_Configure_(const Config_t &pset)
  {
    auto name_combined = pset.get<std::string>("VertexProducer");
    _combined_id=kINVALID_ALGO_ID;
    if (!name_combined.empty()) {
      _combined_id = this->ID(name_combined);
      if (_combined_id == kINVALID_ALGO_ID) {
	LAROCV_CRITICAL() << "Seed ID algorithm name does not exist!" << std::endl;
	throw larbys();
      }
    }

    auto name_particle = pset.get<std::string>("ParticleProducer");
    _particle_id=kINVALID_ALGO_ID;
    if (!name_particle.empty()) {
      _particle_id = this->ID(name_particle);
      if (_particle_id == kINVALID_ALGO_ID) {
	LAROCV_CRITICAL() << "Seed ID algorithm name does not exist!" << std::endl;
	throw larbys();
      }
    }

    _tree = new TTree("ShapeAnalysis","");
    AttachIDs(_tree);
    _tree->Branch("roid"       , &_roid      , "roid/I");
    _tree->Branch("vtxid"      , &_vtxid     , "vtxid/I");
    _tree->Branch("x"          , &_x         , "x/D");
    _tree->Branch("y"          , &_y         , "y/D");
    _tree->Branch("z"          , &_z         , "z/D");
    _tree->Branch("nparticles" , &_nparticles, "nparticles/I");

    _tree->Branch("nplanes_v"         , &_nplanes_v);

    _tree->Branch("length_v"          , &_length_v);
    _tree->Branch("width_v"           , &_width_v);
    _tree->Branch("perimeter_v"       , &_perimeter_v);
    _tree->Branch("area_v"            , &_area_v);
    _tree->Branch("npixel_v"          , &_npixel_v);
    _tree->Branch("qsum_v"            , &_qsum_v);  
    _tree->Branch("track_frac_v"      , &_track_frac_v);
    _tree->Branch("shower_frac_v"     , &_shower_frac_v);
    _tree->Branch("mean_pixel_dist_v" , &_mean_pixel_dist_v);
    _tree->Branch("sigma_pixel_dist_v", &_sigma_pixel_dist_v);
    _tree->Branch("angular_sum_v"     , &_angular_sum_v);
    _tree->Branch("triangle_d_sum_v"  , &_triangle_d_sum_v);

    _tree->Branch("length_max_v"          , &_length_max_v);
    _tree->Branch("width_max_v"           , &_width_max_v);
    _tree->Branch("perimeter_max_v"       , &_perimeter_max_v);
    _tree->Branch("area_max_v"            , &_area_max_v);
    _tree->Branch("npixel_max_v"          , &_npixel_max_v);
    _tree->Branch("qsum_max_v"            , &_qsum_max_v);
    _tree->Branch("track_frac_max_v"      , &_track_frac_max_v);
    _tree->Branch("shower_frac_max_v"     , &_shower_frac_max_v);
    _tree->Branch("mean_pixel_dist_max_v" , &_mean_pixel_dist_max_v);
    _tree->Branch("sigma_pixel_dist_max_v", &_sigma_pixel_dist_max_v);
    _tree->Branch("angular_sum_max_v"     , &_angular_sum_max_v);
    _tree->Branch("triangle_d_max_v"      , &_triangle_d_max_v);

    _tree->Branch("length_min_v"           , &_length_min_v);
    _tree->Branch("width_min_v"            , &_width_min_v);
    _tree->Branch("perimeter_min_v"        , &_perimeter_min_v);
    _tree->Branch("area_min_v"             , &_area_min_v);
    _tree->Branch("npixel_min_v"           , &_npixel_min_v);
    _tree->Branch("qsum_min_v"             , &_qsum_min_v);
    _tree->Branch("track_frac_min_v"       , &_track_frac_min_v);
    _tree->Branch("shower_frac_min_v"      , &_shower_frac_min_v);
    _tree->Branch("mean_pixel_dist_min_v"  , &_mean_pixel_dist_min_v);
    _tree->Branch("sigma_pixel_dist_min_v" , &_sigma_pixel_dist_min_v);
    _tree->Branch("angular_sum_min_v"      , &_angular_sum_min_v);
    _tree->Branch("triangle_d_min_v"       , &_triangle_d_min_v);
     
    _tree->Branch("track_par_max_id"   , &_track_par_max_id   , "track_par_max_id/I");
    _tree->Branch("shower_par_max_id"  , &_shower_par_max_id  , "shower_par_max_id/I");
    _tree->Branch("track_par_max_frac" , &_track_par_max_frac , "track_par_max_frac/F");
    _tree->Branch("shower_par_max_frac", &_shower_par_max_frac, "shower_par_max_frac/F");

    _tree->Branch("par1_type", &_par1_type, "par1_type/I");
    _tree->Branch("par2_type", &_par2_type, "par2_type/I");
    _tree->Branch("par1_frac", &_par1_frac, "par1_frac/F");
    _tree->Branch("par2_frac", &_par2_frac, "par2_frac/F");

    //
    // per plane

    // U
    _tree->Branch("length_U_v"          , &_length_U_v);
    _tree->Branch("width_U_v"           , &_width_U_v);
    _tree->Branch("perimeter_U_v"       , &_perimeter_U_v);
    _tree->Branch("area_U_v"            , &_area_U_v);
    _tree->Branch("npixel_U_v"          , &_npixel_U_v);
    _tree->Branch("qsum_U_v"            , &_qsum_U_v);  
    _tree->Branch("track_frac_U_v"      , &_track_frac_U_v);
    _tree->Branch("shower_frac_U_v"     , &_shower_frac_U_v);
    _tree->Branch("mean_pixel_dist_U_v" , &_mean_pixel_dist_U_v);
    _tree->Branch("sigma_pixel_dist_U_v", &_sigma_pixel_dist_U_v);
    _tree->Branch("angular_sum_U_v"     , &_angular_sum_U_v);
    _tree->Branch("triangle_d_U_v"  , &_triangle_d_U_v);
    
    // V
    _tree->Branch("length_V_v"          , &_length_V_v);
    _tree->Branch("width_V_v"           , &_width_V_v);
    _tree->Branch("perimeter_V_v"       , &_perimeter_V_v);
    _tree->Branch("area_V_v"            , &_area_V_v);
    _tree->Branch("npixel_V_v"          , &_npixel_V_v);
    _tree->Branch("qsum_V_v"            , &_qsum_V_v);  
    _tree->Branch("track_frac_V_v"      , &_track_frac_V_v);
    _tree->Branch("shower_frac_V_v"     , &_shower_frac_V_v);
    _tree->Branch("mean_pixel_dist_V_v" , &_mean_pixel_dist_V_v);
    _tree->Branch("sigma_pixel_dist_V_v", &_sigma_pixel_dist_V_v);
    _tree->Branch("angular_sum_V_v"     , &_angular_sum_V_v);
    _tree->Branch("triangle_d_V_v"  , &_triangle_d_V_v);

    // Y
    _tree->Branch("length_Y_v"          , &_length_Y_v);
    _tree->Branch("width_Y_v"           , &_width_Y_v);
    _tree->Branch("perimeter_Y_v"       , &_perimeter_Y_v);
    _tree->Branch("area_Y_v"            , &_area_Y_v);
    _tree->Branch("npixel_Y_v"          , &_npixel_Y_v);
    _tree->Branch("qsum_Y_v"            , &_qsum_Y_v);  
    _tree->Branch("track_frac_Y_v"      , &_track_frac_Y_v);
    _tree->Branch("shower_frac_Y_v"     , &_shower_frac_Y_v);
    _tree->Branch("mean_pixel_dist_Y_v" , &_mean_pixel_dist_Y_v);
    _tree->Branch("sigma_pixel_dist_Y_v", &_sigma_pixel_dist_Y_v);
    _tree->Branch("angular_sum_Y_v"     , &_angular_sum_Y_v);
    _tree->Branch("triangle_d_Y_v"  , &_triangle_d_Y_v);

    _tree->Branch("vertex_type",&_vertex_type,"vertex_type/I");
    
    _roid = 0;
    
    Register(new data::Info2DArray);    
  }

  
  void ShapeAnalysis::_Process_() {
    LAROCV_INFO() << "start" << std::endl;
    ClearEvent();
    
    if(NextEvent()) _roid=0;
    
    auto& ass_man = AssManager();
    
    // Get images
    auto adc_img_v     = ImageArray(ImageSetID_t::kImageSetWire);
    auto track_img_v   = ImageArray(ImageSetID_t::kImageSetTrack);
    auto shower_img_v  = ImageArray(ImageSetID_t::kImageSetShower);

    // Get Info2D to store particle type
    auto& info2d_arr = AlgoData<data::Info2DArray>(0);
    
    // Get the particle clusters from the previous module, go vertex-by-vertex
    const auto& vtx3d_arr = AlgoData<data::Vertex3DArray>(_combined_id,0);
    const auto& vtx3d_v = vtx3d_arr.as_vector();
    
    const auto& particle_arr = AlgoData<data::ParticleArray>(_particle_id,0);
    const auto& particle_v = particle_arr.as_vector();

    _vtxid = -1;
    for(size_t vtxid = 0; vtxid < vtx3d_v.size(); ++vtxid) {

      ClearVertex();

      const auto& vtx3d = vtx3d_v[vtxid];
      
      auto par_id_v = ass_man.GetManyAss(vtx3d,particle_arr.ID());
      if (par_id_v.empty()) continue;

      _vertex_type = (int) vtx3d.type;
      
      _x = vtx3d.x;
      _y = vtx3d.y;
      _z = vtx3d.z;
      
      _vtxid += 1;

      std::vector<const data::Particle*> par_v;

      for(auto par_id : par_id_v) {
	const auto& par = particle_v[par_id];
	par_v.push_back(&par);
      }

      for(const auto par_ptr : par_v) {
	const auto& par = *par_ptr;
	
	float length    = 0.0;
	float width     = 0.0;
	float perimeter = 0.0;
	float area  = 0.0;
	float npixel = 0.0;
	float qsum = 0.0;
	float track_frac = 0.0;
	float shower_frac = 0.0;
	float mean_pixel_dist = 0.0;
	float sigma_pixel_dist = 0.0;
	float angular_sum = 0.0;
	float triangle_d_sum = 0.0;

	float width_max            = -1.0*kINVALID_FLOAT;
	float length_max           = -1.0*kINVALID_FLOAT;
	float perimeter_max        = -1.0*kINVALID_FLOAT;
	float area_max             = -1.0*kINVALID_FLOAT;
	float npixel_max           = -1.0*kINVALID_FLOAT;
	float qsum_max             = -1.0*kINVALID_FLOAT;
	float track_frac_max       = -1.0*kINVALID_FLOAT;
	float shower_frac_max      = -1.0*kINVALID_FLOAT;
	float mean_pixel_dist_max  = -1.0*kINVALID_FLOAT;
	float sigma_pixel_dist_max = -1.0*kINVALID_FLOAT;
	float angular_sum_max      = -1.0*kINVALID_FLOAT;
	float triangle_d_max       = -1.0*kINVALID_FLOAT;

	float length_min           =  1.0*kINVALID_FLOAT;
	float width_min            =  1.0*kINVALID_FLOAT;
	float perimeter_min        =  1.0*kINVALID_FLOAT;
	float area_min             =  1.0*kINVALID_FLOAT;
	float npixel_min           =  1.0*kINVALID_FLOAT;
	float qsum_min             =  1.0*kINVALID_FLOAT;
	float track_frac_min       =  1.0*kINVALID_FLOAT;
	float shower_frac_min      =  1.0*kINVALID_FLOAT;
	float mean_pixel_dist_min  =  1.0*kINVALID_FLOAT;
	float sigma_pixel_dist_min =  1.0*kINVALID_FLOAT;
	float angular_sum_min      =  1.0*kINVALID_FLOAT;
	float triangle_d_min       =  1.0*kINVALID_FLOAT;
	
	float width_U            = -1.0*kINVALID_FLOAT;
	float length_U           = -1.0*kINVALID_FLOAT;
	float perimeter_U        = -1.0*kINVALID_FLOAT;
	float area_U             = -1.0*kINVALID_FLOAT;
	float npixel_U           = -1.0*kINVALID_FLOAT;
	float qsum_U             = -1.0*kINVALID_FLOAT;
	float track_frac_U       = -1.0*kINVALID_FLOAT;
	float shower_frac_U      = -1.0*kINVALID_FLOAT;
	float mean_pixel_dist_U  = -1.0*kINVALID_FLOAT;
	float sigma_pixel_dist_U = -1.0*kINVALID_FLOAT;
	float angular_sum_U      = -1.0*kINVALID_FLOAT;
	float triangle_d_U       = -1.0*kINVALID_FLOAT;

	float width_V            = -1.0*kINVALID_FLOAT;
	float length_V           = -1.0*kINVALID_FLOAT;
	float perimeter_V        = -1.0*kINVALID_FLOAT;
	float area_V             = -1.0*kINVALID_FLOAT;
	float npixel_V           = -1.0*kINVALID_FLOAT;
	float qsum_V             = -1.0*kINVALID_FLOAT;
	float track_frac_V       = -1.0*kINVALID_FLOAT;
	float shower_frac_V      = -1.0*kINVALID_FLOAT;
	float mean_pixel_dist_V  = -1.0*kINVALID_FLOAT;
	float sigma_pixel_dist_V = -1.0*kINVALID_FLOAT;
	float angular_sum_V      = -1.0*kINVALID_FLOAT;
	float triangle_d_V       = -1.0*kINVALID_FLOAT;

	float width_Y            = -1.0*kINVALID_FLOAT;
	float length_Y           = -1.0*kINVALID_FLOAT;
	float perimeter_Y        = -1.0*kINVALID_FLOAT;
	float area_Y             = -1.0*kINVALID_FLOAT;
	float npixel_Y           = -1.0*kINVALID_FLOAT;
	float qsum_Y             = -1.0*kINVALID_FLOAT;
	float track_frac_Y       = -1.0*kINVALID_FLOAT;
	float shower_frac_Y      = -1.0*kINVALID_FLOAT;
	float mean_pixel_dist_Y  = -1.0*kINVALID_FLOAT;
	float sigma_pixel_dist_Y = -1.0*kINVALID_FLOAT;
	float angular_sum_Y      = -1.0*kINVALID_FLOAT;
	float triangle_d_Y       = -1.0*kINVALID_FLOAT;
	
	int nplanes = 0;
	
	for(size_t plane=0; plane<3; ++plane) {
	  
	  const auto& pcluster = par._par_v[plane];
	  const auto& pctor = pcluster._ctor;
	  if (pctor.empty()) continue;
	  
	  nplanes += 1;
	  
	  auto& adc_img    = adc_img_v[plane];
	  auto& track_img  = track_img_v[plane];
	  auto& shower_img = shower_img_v[plane];
	    
	  PixelChunk pchunk(pctor,adc_img,track_img,shower_img);
	  length           += pchunk.length;
	  width            += pchunk.width;
	  perimeter        += pchunk.perimeter;
	  area             += pchunk.area;
	  npixel           += pchunk.npixel;
	  qsum             += pchunk.qsum;
	  track_frac       += pchunk.track_frac;
	  shower_frac      += pchunk.shower_frac;
	  mean_pixel_dist  += pchunk.mean_pixel_dist;
	  sigma_pixel_dist += pchunk.sigma_pixel_dist;
	  angular_sum      += pchunk.angular_sum;


	  // Estimate linearity from 3 point triangle
	  const auto& vtx2d = vtx3d.vtx2d_v.at(plane).pt;
	  auto valid_end = pchunk.EstimateStartEndPixel(vtx2d,adc_img,10);
	  if (!valid_end) LAROCV_WARNING() << "Invalid start or end point" << std::endl;
	  auto valid_trunk = pchunk.EstimateTrunkPCA(vtx2d,adc_img,15,10);
	  if (!valid_trunk) LAROCV_WARNING() << "Invalid trunk PCA determined" << std::endl;

	  geo2d::Vector<float> ret1,ret2;
	  float dist = geo2d::ClosestPoint(pchunk.trunkPCA,pchunk.end_pt,ret1,ret2);
	  if (std::isinf(dist))
	    dist = kINVALID_FLOAT;

	  triangle_d_sum += dist;

	  if (dist > triangle_d_max && dist != kINVALID_FLOAT)
	    triangle_d_max = dist;

	  if (dist < triangle_d_min && dist != kINVALID_FLOAT)
	    triangle_d_min = dist;

	  if (pchunk.length      > length_max)      length_max       = pchunk.length;
	  if (pchunk.width       > width_max)       width_max        = pchunk.width;
	  if (pchunk.perimeter   > perimeter_max)   perimeter_max    = pchunk.perimeter;
	  if (pchunk.area        > area_max)        area_max         = pchunk.area;
	  if (pchunk.npixel      > npixel_max)      npixel_max       = pchunk.npixel;
	  if (pchunk.qsum        > qsum_max)        qsum_max         = pchunk.qsum;
	  if (pchunk.track_frac  > track_frac_max)  track_frac_max   = pchunk.track_frac;
	  if (pchunk.shower_frac > shower_frac_max) shower_frac_max  = pchunk.shower_frac;

	  if (pchunk.mean_pixel_dist  > mean_pixel_dist_max)  mean_pixel_dist_max  = pchunk.mean_pixel_dist;
	  if (pchunk.sigma_pixel_dist > sigma_pixel_dist_max) sigma_pixel_dist_max = pchunk.sigma_pixel_dist;


	  if (pchunk.length      < length_min)      length_min       = pchunk.length;
	  if (pchunk.width       < width_min)       width_min        = pchunk.width;
	  if (pchunk.perimeter   < perimeter_min)   perimeter_min    = pchunk.perimeter;
	  if (pchunk.area        < area_min)        area_min         = pchunk.area;
	  if (pchunk.npixel      < npixel_min)      npixel_min       = pchunk.npixel;
	  if (pchunk.qsum        < qsum_min)        qsum_min         = pchunk.qsum;
	  if (pchunk.track_frac  < track_frac_min)  track_frac_min   = pchunk.track_frac;
	  if (pchunk.shower_frac < shower_frac_min) shower_frac_min  = pchunk.shower_frac;

	  if (pchunk.mean_pixel_dist  < mean_pixel_dist_min)  mean_pixel_dist_min  = pchunk.mean_pixel_dist;
	  if (pchunk.sigma_pixel_dist < sigma_pixel_dist_min) sigma_pixel_dist_min = pchunk.sigma_pixel_dist;
	  	  
	  switch(plane) {
	    
	  case 0:
	    width_U            = pchunk.width;
	    length_U           = pchunk.length;
	    perimeter_U        = pchunk.perimeter;
	    area_U             = pchunk.area;
	    npixel_U           = pchunk.npixel;
	    qsum_U             = pchunk.qsum;
	    track_frac_U       = pchunk.track_frac;
	    shower_frac_U      = pchunk.shower_frac;
	    mean_pixel_dist_U  = pchunk.mean_pixel_dist;
	    sigma_pixel_dist_U = pchunk.sigma_pixel_dist;
	    angular_sum_U      = pchunk.angular_sum;
	    triangle_d_U       = dist;
	    break;

	  case 1:
	    width_V            = pchunk.width;
	    length_V           = pchunk.length;
	    perimeter_V        = pchunk.perimeter;
	    area_V             = pchunk.area;
	    npixel_V           = pchunk.npixel;
	    qsum_V             = pchunk.qsum;
	    track_frac_V       = pchunk.track_frac;
	    shower_frac_V      = pchunk.shower_frac;
	    mean_pixel_dist_V  = pchunk.mean_pixel_dist;
	    sigma_pixel_dist_V = pchunk.sigma_pixel_dist;
	    angular_sum_V      = pchunk.angular_sum;
	    triangle_d_V       = dist;
	    break;

	  case 2:
	    width_Y            = pchunk.width;
	    length_Y           = pchunk.length;
	    perimeter_Y        = pchunk.perimeter;
	    area_Y             = pchunk.area;
	    npixel_Y           = pchunk.npixel;
	    qsum_Y             = pchunk.qsum;
	    track_frac_Y       = pchunk.track_frac;
	    shower_frac_Y      = pchunk.shower_frac;
	    mean_pixel_dist_Y  = pchunk.mean_pixel_dist;
	    sigma_pixel_dist_Y = pchunk.sigma_pixel_dist;
	    angular_sum_Y      = pchunk.angular_sum;
	    triangle_d_Y       = dist;
	    break;
	    
	  default:
	    throw larbys("plane not handled, die");
	  }

	  
	} // end plane
	
	_length_v           . push_back(length);
	_width_v            . push_back(width);
	_perimeter_v        . push_back(perimeter);
	_area_v             . push_back(area);
	_npixel_v           . push_back(npixel);
	_qsum_v             . push_back(qsum);
	_track_frac_v       . push_back(track_frac);
	_shower_frac_v      . push_back(shower_frac);
	_mean_pixel_dist_v  . push_back(mean_pixel_dist);
	_sigma_pixel_dist_v . push_back(sigma_pixel_dist);
	_angular_sum_v      . push_back(angular_sum);
	_triangle_d_sum_v   . push_back(triangle_d_sum);

	_length_min_v           . push_back(length_min);
	_width_min_v            . push_back(width_min);
	_perimeter_min_v        . push_back(perimeter_min);
	_area_min_v             . push_back(area_min);
	_npixel_min_v           . push_back(npixel_min);
	_qsum_min_v             . push_back(qsum_min);
	_track_frac_min_v       . push_back(track_frac_min);
	_shower_frac_min_v      . push_back(shower_frac_min);
	_mean_pixel_dist_min_v  . push_back(mean_pixel_dist_min);
	_sigma_pixel_dist_min_v . push_back(sigma_pixel_dist_min);
	_angular_sum_min_v      . push_back(angular_sum_min);
	_triangle_d_min_v       . push_back(triangle_d_min);

	_length_max_v           . push_back(length_max);
	_width_max_v            . push_back(width_max);
	_perimeter_max_v        . push_back(perimeter_max);
	_area_max_v             . push_back(area_max);
	_npixel_max_v           . push_back(npixel_max);
	_qsum_max_v             . push_back(qsum_max);
	_track_frac_max_v       . push_back(track_frac_max);
	_shower_frac_max_v      . push_back(shower_frac_max);
	_mean_pixel_dist_max_v  . push_back(mean_pixel_dist_max);
	_sigma_pixel_dist_max_v . push_back(sigma_pixel_dist_max);
	_angular_sum_max_v      . push_back(angular_sum_max);
	_triangle_d_max_v       . push_back(triangle_d_max);

	_length_U_v           . push_back(length_U);
	_width_U_v            . push_back(width_U);
	_perimeter_U_v        . push_back(perimeter_U);
	_area_U_v             . push_back(area_U);
	_npixel_U_v           . push_back(npixel_U);
	_qsum_U_v             . push_back(qsum_U);
	_track_frac_U_v       . push_back(track_frac_U);
	_shower_frac_U_v      . push_back(shower_frac_U);
	_mean_pixel_dist_U_v  . push_back(mean_pixel_dist_U);
	_sigma_pixel_dist_U_v . push_back(sigma_pixel_dist_U);
	_angular_sum_U_v      . push_back(angular_sum_U);
	_triangle_d_U_v       . push_back(triangle_d_U);

	_length_V_v           . push_back(length_V);
	_width_V_v            . push_back(width_V);
	_perimeter_V_v        . push_back(perimeter_V);
	_area_V_v             . push_back(area_V);
	_npixel_V_v           . push_back(npixel_V);
	_qsum_V_v             . push_back(qsum_V);
	_track_frac_V_v       . push_back(track_frac_V);
	_shower_frac_V_v      . push_back(shower_frac_V);
	_mean_pixel_dist_V_v  . push_back(mean_pixel_dist_V);
	_sigma_pixel_dist_V_v . push_back(sigma_pixel_dist_V);
	_angular_sum_V_v      . push_back(angular_sum_V);
	_triangle_d_V_v       . push_back(triangle_d_V);

	_length_Y_v           . push_back(length_Y);
	_width_Y_v            . push_back(width_Y);
	_perimeter_Y_v        . push_back(perimeter_Y);
	_area_Y_v             . push_back(area_Y);
	_npixel_Y_v           . push_back(npixel_Y);
	_qsum_Y_v             . push_back(qsum_Y);
	_track_frac_Y_v       . push_back(track_frac_Y);
	_shower_frac_Y_v      . push_back(shower_frac_Y);
	_mean_pixel_dist_Y_v  . push_back(mean_pixel_dist_Y);
	_sigma_pixel_dist_Y_v . push_back(sigma_pixel_dist_Y);
	_angular_sum_Y_v      . push_back(angular_sum_Y);
	_triangle_d_Y_v       . push_back(triangle_d_Y);

	_nplanes_v.push_back(nplanes);
	
	  
      } // end particle
      _nparticles = par_id_v.size();

      auto track_iter  = std::max_element(_track_frac_v.begin(),_track_frac_v.end());
      auto shower_iter = std::max_element(_shower_frac_v.begin(),_shower_frac_v.end());

      if (track_iter  == _track_frac_v.end())  throw larbys("Bad Track found");
      if (shower_iter == _shower_frac_v.end()) throw larbys("Bad Shower found");
      
      _track_par_max_id  = track_iter  - _track_frac_v.begin();
      _shower_par_max_id = shower_iter - _shower_frac_v.begin();
      
      _track_par_max_frac  = *track_iter;
      _shower_par_max_frac = *shower_iter;

      std::vector<int> par_type_v(_nparticles,-1.0 * kINVALID_INT);
      std::vector<float> par_frac_v(_nparticles,-1.0 * kINVALID_FLOAT);

      for(size_t parid=0; parid< (size_t)_nparticles; parid++) {
	auto& par_type = par_type_v[parid];
	auto& par_frac = par_frac_v[parid];

	auto track_frac  = _track_frac_v[parid];
	auto shower_frac = _shower_frac_v[parid];

	data::Info2D info2d;
	FillInfo2D(parid,info2d);

	if (track_frac >= shower_frac) {
	  par_type = 1;
	  par_frac = track_frac;
	  info2d.ptype = data::ParticleCluster::ParticleType_t::kTrack;
	}
	else {
	  par_type = 2;
	  par_frac = shower_frac;
	  info2d.ptype = data::ParticleCluster::ParticleType_t::kShower;
	}

	info2d_arr.emplace_back(std::move(info2d));
	AssociateOne(info2d_arr.as_vector().back(),*(par_v[parid]));
      }

      _par1_type = par_type_v.front();
      _par2_type = par_type_v.back();

      _par1_frac = par_frac_v.front();
      _par2_frac = par_frac_v.back();
      
      _tree->Fill();
    } // end this vertex

    _roid += 1;
    LAROCV_INFO() << "end" << std::endl;
  }


  void ShapeAnalysis::ClearEvent() {
    _vtxid = kINVALID_INT;
    ClearVertex();
  }
  
  void ShapeAnalysis::ClearVertex() {

    _x = kINVALID_DOUBLE;
    _y = kINVALID_DOUBLE;
    _z = kINVALID_DOUBLE;

    _nparticles = kINVALID_INT;

    _nplanes_v.clear();

    _length_v.clear();
    _width_v.clear();
    _perimeter_v.clear();
    _area_v.clear();
    _npixel_v.clear();
    _qsum_v.clear();
    _track_frac_v.clear();
    _shower_frac_v.clear();
    _mean_pixel_dist_v.clear();
    _sigma_pixel_dist_v.clear();
    _angular_sum_v.clear();
    _triangle_d_sum_v.clear();

    _length_max_v.clear();
    _width_max_v.clear();
    _perimeter_max_v.clear();
    _area_max_v.clear();
    _npixel_max_v.clear();
    _qsum_max_v.clear();
    _track_frac_max_v.clear();
    _shower_frac_max_v.clear();
    _mean_pixel_dist_max_v.clear();
    _sigma_pixel_dist_max_v.clear();
    _angular_sum_max_v.clear();
    _triangle_d_max_v.clear();

    _length_min_v.clear();
    _width_min_v.clear();
    _perimeter_min_v.clear();
    _area_min_v.clear();
    _npixel_min_v.clear();
    _qsum_min_v.clear();
    _track_frac_min_v.clear();
    _shower_frac_min_v.clear();
    _mean_pixel_dist_min_v.clear();
    _sigma_pixel_dist_min_v.clear();
    _angular_sum_min_v.clear();
    _triangle_d_min_v.clear();

    _length_U_v.clear();
    _width_U_v.clear();
    _perimeter_U_v.clear();
    _area_U_v.clear();
    _npixel_U_v.clear();
    _qsum_U_v.clear();
    _track_frac_U_v.clear();
    _shower_frac_U_v.clear();
    _mean_pixel_dist_U_v.clear();
    _sigma_pixel_dist_U_v.clear();
    _angular_sum_U_v.clear();
    _triangle_d_U_v.clear();

    _length_V_v.clear();
    _width_V_v.clear();
    _perimeter_V_v.clear();
    _area_V_v.clear();
    _npixel_V_v.clear();
    _qsum_V_v.clear();
    _track_frac_V_v.clear();
    _shower_frac_V_v.clear();
    _mean_pixel_dist_V_v.clear();
    _sigma_pixel_dist_V_v.clear();
    _angular_sum_V_v.clear();
    _triangle_d_V_v.clear();

    _length_Y_v.clear();
    _width_Y_v.clear();
    _perimeter_Y_v.clear();
    _area_Y_v.clear();
    _npixel_Y_v.clear();
    _qsum_Y_v.clear();
    _track_frac_Y_v.clear();
    _shower_frac_Y_v.clear();
    _mean_pixel_dist_Y_v.clear();
    _sigma_pixel_dist_Y_v.clear();
    _angular_sum_Y_v.clear();
    _triangle_d_Y_v.clear();
    
    _track_par_max_id = kINVALID_INT;
    _shower_par_max_id = kINVALID_INT;
    
    _track_par_max_frac = kINVALID_FLOAT;
    _shower_par_max_frac = kINVALID_FLOAT;
    
    _par1_type = kINVALID_INT;
    _par2_type = kINVALID_INT;
    
    _par1_frac = kINVALID_FLOAT;
    _par2_frac = kINVALID_FLOAT;
    
    _vertex_type = kINVALID_INT;
  }

  void ShapeAnalysis::FillInfo2D(size_t parid, data::Info2D& info) {

    assert(parid < _nplanes_v.size());
    
    info.nplanes = _nplanes_v[parid];
    info.length = _length_v[parid];
    info.width = _width_v[parid];
    info.perimeter = _perimeter_v[parid];
    info.area = _area_v[parid];
    info.npixel = _npixel_v[parid];
    info.qsum = _qsum_v[parid];
    info.track_frac = _track_frac_v[parid];
    info.shower_frac = _shower_frac_v[parid];
    info.mean_pixel_dist = _mean_pixel_dist_v[parid];
    info.sigma_pixel_dist = _sigma_pixel_dist_v[parid];
    info.angular_sum = _angular_sum_v[parid];
    info.triangle_d_sum = _triangle_d_sum_v[parid];

    info.length_max = _length_max_v[parid];
    info.width_max = _width_max_v[parid];
    info.perimeter_max = _perimeter_max_v[parid];
    info.area_max = _area_max_v[parid];
    info.npixel_max = _npixel_max_v[parid];
    info.qsum_max = _qsum_max_v[parid];
    info.track_frac_max = _track_frac_max_v[parid];
    info.shower_frac_max = _shower_frac_max_v[parid];
    info.mean_pixel_dist_max = _mean_pixel_dist_max_v[parid];
    info.sigma_pixel_dist_max = _sigma_pixel_dist_max_v[parid];
    info.angular_sum_max = _angular_sum_max_v[parid];
    info.triangle_d_max = _triangle_d_max_v[parid];

    info.length_min = _length_min_v[parid];
    info.width_min = _width_min_v[parid];
    info.perimeter_min = _perimeter_min_v[parid];
    info.area_min = _area_min_v[parid];
    info.npixel_min = _npixel_min_v[parid];
    info.qsum_min = _qsum_min_v[parid];
    info.track_frac_min = _track_frac_min_v[parid];
    info.shower_frac_min = _shower_frac_min_v[parid];
    info.mean_pixel_dist_min = _mean_pixel_dist_min_v[parid];
    info.sigma_pixel_dist_min = _sigma_pixel_dist_min_v[parid];
    info.angular_sum_min = _angular_sum_min_v[parid];
    info.triangle_d_min = _triangle_d_min_v[parid];
  }
  
}
#endif
